﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace ELTE.Net.AsyncTcpCommunication
{
    /// <summary>
    /// Kliens oldali hálózati adatok csomagoló típusa.
    /// </summary>
    class ClientNetworkData
    {
        public TcpClient Client { get; set; }
        public Byte[] BufferArray { get; set; }
        public NetworkStream Stream { get; set; }
        public Int32 ClientId { get; set; }
        public Int32 Counter { get; set; }
    }

    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Booting server...");
            Server server = new Server();
            Thread.Sleep(1000); // várakoztatjuk a szálat

            
            Console.WriteLine("Booting clients...");
            for (Int32 i = 0; i < 10; i++)
            {
                Thread.Sleep(500);

                TcpClient client = new TcpClient();
                client.BeginConnect(IPAddress.Parse("127.0.0.1"), 3000, new AsyncCallback(ClientConnected), new ClientNetworkData() { Client = client, ClientId = i });
            }

            Thread.Sleep(10000); // kell némi várakoztatás, hogy elinduljon a folyamat
            
        }

        private static void ClientConnected(IAsyncResult asyncResult)
        {
            TcpClient client = ((ClientNetworkData)asyncResult.AsyncState).Client; // az állapot tartalmazza a kliens objektumot
            Int32 id = ((ClientNetworkData)asyncResult.AsyncState).ClientId; // és az azonosítót
            client.EndConnect(asyncResult); // kapcsolódás vége

            NetworkStream networkStream = client.GetStream();

            Byte[] buffer = System.Text.Encoding.UTF8.GetBytes("Hello! I'm client nr. " + id + "!");

            networkStream.BeginWrite(buffer, 0, buffer.Length, new AsyncCallback(MessageSent), new ClientNetworkData() { BufferArray = buffer, Stream = networkStream, ClientId = id, Counter = 0 }); // küldés elkezdése
        }

        /// <summary>
        /// Üzenetfogadás a szervertől.
        /// </summary>
        /// <param name="asyncResult">A kliens adatfolyam és a folyamat állapota.</param>
        private static void MessageReceived(IAsyncResult asyncResult)
        {
            NetworkStream networkStream = ((ClientNetworkData)asyncResult.AsyncState).Stream; // lekérjük az adatokat
            Byte[] buffer = ((ClientNetworkData)asyncResult.AsyncState).BufferArray;
            Int32 id = ((ClientNetworkData)asyncResult.AsyncState).ClientId;
            Int32 counter = ((ClientNetworkData)asyncResult.AsyncState).Counter;

            Int32 byteCount = networkStream.EndRead(asyncResult); // olvasás vége, visszakapjuk az olvasott bájtok számát

            String message = System.Text.Encoding.UTF8.GetString(buffer, 0, byteCount); // üzenet dekódolása
            Console.WriteLine("Client " + id + ", receive: " + message);

            if (counter < 10) // ha még nem küldtünk el 10 üzenetet
            {
                Thread.Sleep(1000);
                counter++;

                buffer = System.Text.Encoding.UTF8.GetBytes(counter.ToString()); // byte-ok kiolvasása
                networkStream.BeginWrite(buffer, 0, buffer.Length, new AsyncCallback(MessageSent), new ClientNetworkData() { BufferArray = buffer, Stream = networkStream, ClientId = id, Counter = counter }); // küldés elkezdése
            }
            else
            {
                networkStream.Close(); // különben bezárjuk a csatornát
            }
        }

        /// <summary>
        /// Üzenetküldés a szervernek.
        /// </summary>
        /// <param name="asyncResult">A kliens adatfolyam és a folyamat állapota.</param>
        private static void MessageSent(IAsyncResult asyncResult)
        {
            NetworkStream networkStream = ((ClientNetworkData)asyncResult.AsyncState).Stream; // lekérjük az adatokat
            Byte[] buffer = ((ClientNetworkData)asyncResult.AsyncState).BufferArray;
            Int32 id = ((ClientNetworkData)asyncResult.AsyncState).ClientId;
            Int32 counter = ((ClientNetworkData)asyncResult.AsyncState).Counter;

            networkStream.EndWrite(asyncResult); // írás vége

            buffer = new Byte[256]; // új puffertömb az olvasáshoz
            networkStream.BeginRead(buffer, 0, 256, new AsyncCallback(MessageReceived), new ClientNetworkData() { BufferArray = buffer, Stream = networkStream, ClientId = id, Counter = counter }); // olvasás elkezdése
        }
    }
}
