﻿using System;
using System.Net;
using System.Net.Sockets;

namespace ELTE.Net.AsyncTcpCommunication
{
    /// <summary>
    /// Szerver oldali hálózati adatok csomagoló típusa.
    /// </summary>
    class ServerNetworkData
    {
        public Byte[] BufferArray { get; set; }
        public NetworkStream Stream { get; set; }
        public Boolean IsFirst { get; set; }
    }

    /// <summary>
    /// Szerver típusa.
    /// </summary>
    class Server
    {
        public Server()
        {
            TcpListener listener = new TcpListener(IPAddress.Any, 3000); // TCP szerver, amely kliensek csatlakozására vár
            listener.Start(); // figyelés indítása
            listener.BeginAcceptTcpClient(new AsyncCallback(ClientConnected), listener);
        }

        /// <summary>
        /// Kliens csatlakozásának kezelése.
        /// </summary>
        /// <param name="asyncResult">A szerver és a folyamat állapota.</param>
        private void ClientConnected(IAsyncResult asyncResult)
        {
            TcpListener listener = (TcpListener)asyncResult.AsyncState; // az állapot tartalmazza a szerver objektumot
            TcpClient client = listener.EndAcceptTcpClient(asyncResult); // véglegesítjük a műveletet, és lekérjük az eredményt

            NetworkStream networkStream = client.GetStream();

            Byte[] buffer = new Byte[256]; // az adatokat egy tömbbe helyezzük
            networkStream.BeginRead(buffer, 0, 256, new AsyncCallback(MessageReceived), new ServerNetworkData() { BufferArray = buffer, Stream = networkStream, IsFirst = true }); // olvasás elkezdése

            listener.BeginAcceptTcpClient(new AsyncCallback(ClientConnected), listener); // jöhet a következő kliens
        }

        /// <summary>
        /// Üzenetfogadás a klienstől.
        /// </summary>
        /// <param name="asyncResult">A kliens adatfolyam és a folyamat állapota.</param>
        private void MessageReceived(IAsyncResult asyncResult)
        {
            NetworkStream networkStream = ((ServerNetworkData)asyncResult.AsyncState).Stream; // lekérjük az adatokat
            Byte[] buffer = ((ServerNetworkData)asyncResult.AsyncState).BufferArray;

            Int32 byteCount = networkStream.EndRead(asyncResult); // olvasás vége, visszakapjuk az olvasott bájtok számát

            String message = System.Text.Encoding.UTF8.GetString(buffer, 0, byteCount); // üzenet dekódolása
            Console.WriteLine("Server, receive: " + message);

            if (((ServerNetworkData)asyncResult.AsyncState).IsFirst) // ha első alkalommal fogadtunk
            {
                buffer = System.Text.Encoding.UTF8.GetBytes("Hello Client!"); // byte-ok kiolvasása
            }
            else
            {
                buffer = System.Text.Encoding.UTF8.GetBytes("Ok, got: " + message);
            }

            try
            {
                networkStream.BeginWrite(buffer, 0, buffer.Length, new AsyncCallback(MessageSent), new ServerNetworkData() { BufferArray = buffer, Stream = networkStream, IsFirst = false }); // küldés elkezdése
            }
            catch (Exception) // ha közben a kliens befejezi a kapcsolatot
            {
            }
        }

        /// <summary>
        /// Üzenetküldés a kliensnek.
        /// </summary>
        /// <param name="asyncResult">A kliens adatfolyam és a folyamat állapota.</param>
        private void MessageSent(IAsyncResult asyncResult)
        {
            NetworkStream networkStream = ((ServerNetworkData)asyncResult.AsyncState).Stream; // lekérjük az adatokat
            Byte[] buffer = ((ServerNetworkData)asyncResult.AsyncState).BufferArray;
            networkStream.EndWrite(asyncResult); // írás vége

            try
            {
                buffer = new Byte[256]; // új puffertömb az olvasáshoz
                networkStream.BeginRead(buffer, 0, 256, new AsyncCallback(MessageReceived), new ServerNetworkData() { BufferArray = buffer, Stream = networkStream, IsFirst = false }); // olvasás elkezdése
            }
            catch (Exception) // ha közben a kliens befejezi a kapcsolatot
            {
                Console.WriteLine("Server: client disconnected.");
            }
        }
    }
}
