﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace ELTE.Net.SyncTcpCommunication
{
    /// <summary>
    /// Szerver típusa.
    /// </summary>
    class Server
    {
        private TcpListener _TcpListener;
        private Thread _ListenThread;

        public Server()
        {
            _TcpListener = new TcpListener(IPAddress.Any, 3000); // TCP szerver, amely kliensek csatlakozására vár
            _ListenThread = new Thread(ListenForClients); // a hallgatózás párhuzamosan fut
            _ListenThread.Start();
        }

        /// <summary>
        /// A kliensek figyelése.
        /// </summary>
        private void ListenForClients()
        {
            _TcpListener.Start();

            while (true)
            {
                TcpClient client = _TcpListener.AcceptTcpClient(); // várakozás csatlakozásig

                Thread clientThread = new Thread(HandleCommunication);
                clientThread.Start(client); // kezelés párhuzamos szálának indítása, paraméterben megkapja a klienst
            }
        }

        /// <summary>
        /// A kliens kommunikáció lebonyolítása.
        /// </summary>
        /// <param name="client">Kliens objektum.</param>
        private void HandleCommunication(Object client)
        {
            TcpClient tcpClient = (TcpClient)client;
            NetworkStream networkStream = tcpClient.GetStream(); // TCP adatfolyam lekérése
            StreamReader reader = new StreamReader(networkStream);
            StreamWriter writer = new StreamWriter(networkStream);

            try
            {
                String message = reader.ReadLine();
                Console.WriteLine("Server, receive: " + message);
                writer.WriteLine("Hello Client!");
                writer.Flush();
                
                while (true)
                {
                    message = reader.ReadLine();
                    Console.WriteLine("Server, receive: " + message);
                    writer.WriteLine("Ok, got: " + message);
                    writer.Flush();
                }
            }
            catch (Exception) // sikertelen olvasás, ekkor megszakadt a folyam
            {
                Console.WriteLine("Server: client disconnected.");
                if (tcpClient.Connected)
                    tcpClient.Close(); // kapccsolat bezárása
            }
        }
    }
}
