﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Windows;

namespace ELTE.Net.ChatRoom.Udp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private IPEndPoint _RemoteEndPoint; // távoli végpont
        private UdpClient _UdpClient; // UDP kliens

        public MainWindow()
        {
            InitializeComponent();

            _RemoteEndPoint = new IPEndPoint(IPAddress.Any, 0);
        }

        private void MessageReceived(IAsyncResult asyncResult)
        {
            try
            {
                Byte[] messageBytes = _UdpClient.EndReceive(asyncResult, ref _RemoteEndPoint); // üzenet fogadása
                if (messageBytes.Length > 0)
                {

                    String message = System.Text.Encoding.UTF8.GetString(messageBytes); // kiolvasás szövegként

                    _ListBoxMessages.Dispatcher.Invoke(new Action(() => _ListBoxMessages.Items.Insert(0, message)));

                    _UdpClient.BeginReceive(new AsyncCallback(MessageReceived), _UdpClient); // üzenetfogadás kezdete
                }
                else // ha nem sikerült adatot fogadni, akkor valószínűleg vége a kapcsolatnak
                {
                    _ListBoxMessages.Dispatcher.Invoke(new Action(() => _ListBoxMessages.Items.Insert(0, "Connection aborted.")));
                }
            }
            catch (Exception)
            {
            }
        }

        private void AddMessage(String message)
        {
           ; // üzenet megjelenítése a listában
        }

        private void ButtonConnect_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Int32 remotePort = Int32.Parse(_TextRemotePort.Text); // távoli port számmá konvertálása, ha lehetséges
                Int32 localPort = Int32.Parse(_TextLocalPort.Text); // lokális port számmá konvertálása, ha lehetséges
                _RemoteEndPoint = new IPEndPoint(IPAddress.Parse(_TextAddress.Text), remotePort);
                _UdpClient = new UdpClient(localPort); // lokális portszám megadása
                _UdpClient.BeginReceive(new AsyncCallback(MessageReceived), _UdpClient); // üzenetfogadás kezdete

                _ListBoxMessages.Items.Insert(0, "Listening on local port " + localPort + "...");
                _ListBoxMessages.Items.Insert(0, "Receiving messages from " + _TextAddress.Text + ":" + remotePort + "...");
            }
            catch (FormatException)
            {
                MessageBox.Show("One of the given port number is not a valid value.", "Invalid port number!", MessageBoxButton.OK, MessageBoxImage.Warning);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error occured during message receive!", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void ButtonSend_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Byte[] messageBytes = System.Text.Encoding.UTF8.GetBytes("[" + _TextName.Text + "] " + _TextMessage.Text);
                _UdpClient.Send(messageBytes, messageBytes.Length, _RemoteEndPoint); // üzenet küldése a megadott végpontra
                _TextMessage.Text = String.Empty;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error occured during message sending!", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
    }
}
