﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Windows;
using System.Windows.Input;
using System.Windows.Threading;

namespace ELTE.Net.ChatRoom.Tcp.Client
{
    public partial class MainWindow : Window
    {
        private Socket _ChatSocket;
        private Byte[] _MessageBuffer;

        public MainWindow()
        {
            InitializeComponent();

            _MessageBuffer = new Byte[1024];
        }

        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            if (_ChatSocket != null && _ChatSocket.Connected) // bezárás előtt le kell kapcsolni a csatlakozást
            {
                _ChatSocket.Shutdown(SocketShutdown.Both);
                _ChatSocket.Close();
            }
            base.OnClosing(e);
        }

        private void Receive_Completed(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError == SocketError.Success)
            {
                String message = System.Text.Encoding.UTF8.GetString(e.Buffer, 0, e.BytesTransferred); // kiolvasás a pufferből szövegként

                _ListBoxMessages.Dispatcher.Invoke(new Action(() => _ListBoxMessages.Items.Insert(0, message)));

                e.Dispose(); // régi eseményargumentum megsemmisítése

                SocketAsyncEventArgs socketAsyncEventArgs = new SocketAsyncEventArgs();
                socketAsyncEventArgs.RemoteEndPoint = e.RemoteEndPoint;
                socketAsyncEventArgs.SetBuffer(new Byte[1024], 0, 1024);
                socketAsyncEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Receive_Completed);
                _ChatSocket.ReceiveAsync(socketAsyncEventArgs);
                // fogadás aszinkron módon
            }
            else // ha nem sikerült adatot fogadni, akkor valószínűleg vége a kapcsolatnak
            {
                _ListBoxMessages.Dispatcher.BeginInvoke(new Action(() => _ListBoxMessages.Items.Insert(0, "Client disconnected.")));
                    // szálbiztos kiírás a felületre

                _ChatSocket.Shutdown(SocketShutdown.Both);
                System.Threading.Thread.Sleep(10);
                _ChatSocket.Close();
            }
        }

        private void ButtonConnect_Click(object sender, RoutedEventArgs e)
        {
            Cursor= Cursors.Wait; // homokóra beállítása kurzornak
            try
            {
                if (_ChatSocket != null && _ChatSocket.Connected) // socket bezárása, ha még nyitva van
                {
                    _ChatSocket.Shutdown(SocketShutdown.Both); // mindkét oldali leállítás
                    System.Threading.Thread.Sleep(10);
                    _ChatSocket.Close();
                }
                _ChatSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                    // TCP socket létrehozása
                IPEndPoint serverEndPoint = new IPEndPoint(IPAddress.Parse(_TextAddress.Text), 399);
                    // távoli végpont
                _ListBoxMessages.Items.Insert(0, "Starting connection to " + _TextAddress.Text + "...");
                _ChatSocket.Blocking = false; // nem blokkoló mód

                SocketAsyncEventArgs socketAsyncEventArgs = new SocketAsyncEventArgs(); // argumentumok a kapcsolódáshoz
                socketAsyncEventArgs.RemoteEndPoint = serverEndPoint; // távoli végpont megadása
                socketAsyncEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Connection_Completed);
                    // a kapcsolódás végeztével futtatandó eseménykezelő
                _ChatSocket.ConnectAsync(socketAsyncEventArgs);
                    // kapcsolódás aszinkron módon
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error occured during connection!", MessageBoxButton.OK, MessageBoxImage.Error);
            }

            Cursor = Cursors.Arrow; // kurzor visszaállítása
        }

        private void Connection_Completed(object sender, SocketAsyncEventArgs e)
        {
            if (e.SocketError == SocketError.Success) // ellenőrizzük a művelet sikerességét
            {
                _ListBoxMessages.Dispatcher.Invoke(new Action(() => _ListBoxMessages.Items.Insert(0, "Connected to " + _TextAddress.Text + "!")));

                e.Dispose();

                SocketAsyncEventArgs socketAsyncEventArgs = new SocketAsyncEventArgs();
                socketAsyncEventArgs.RemoteEndPoint = e.RemoteEndPoint;
                socketAsyncEventArgs.SetBuffer(new Byte[1024], 0, 1024); // puffer a fogadáshoz
                socketAsyncEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Receive_Completed);
                _ChatSocket.ReceiveAsync(socketAsyncEventArgs);
                    // fogadás aszinkron módon
            }
            else
            {
                Dispatcher.Invoke(new Action(() => _ListBoxMessages.Items.Insert(0, "Error occured during connection!")));
            }
        }

        private void ButtonSend_Click(object sender, RoutedEventArgs e)
        {
            if (_ChatSocket == null || !_ChatSocket.Connected)
            {
                MessageBox.Show("Please connect before sending message.", "No connection!", MessageBoxButton.OK, MessageBoxImage.Stop);
                return;
            }

            try
            {
                Byte[] messageBytes = System.Text.Encoding.UTF8.GetBytes("[" + _TextName.Text + "] " + _TextMessage.Text); 
                    // üzenet átalakítása byte tömbbé
                _ChatSocket.Send(messageBytes, messageBytes.Length, 0); // üzenet küldése szinkron módon
                _TextMessage.Text = String.Empty;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error occured during sending!", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
    }
}
