﻿using System;
using System.Collections.ObjectModel;
using System.Windows.Input;
using System.Windows.Threading;
using ELTE.Net.ChatRoom.Tcp.ClientInViewModel.Model;

namespace ELTE.Net.ChatRoom.Tcp.ClientInViewModel.ViewModel
{
    /// <summary>
    /// Hálózati kapcsolat nézetmodellje.
    /// </summary>
    public class NetworkManagerViewModel : ViewModelBase
    {
        private NetworkManager _Manager;
        private RelayCommand _SendCommand;
        private RelayCommand _ConnectCommand;

        /// <summary>
        /// Üzeneteket tartalmazó felügyelt gyűjtemény lekérdezése.
        /// </summary>
        public ObservableCollection<String> Messages { get; private set; }
        /// <summary>
        /// Felhasználónév lekérdezése, vagy beállítása.
        /// </summary>
        public String UserName { get; set; }
        /// <summary>
        /// Távoli végpont címének lekérdezése, vagy beállítása.
        /// </summary>
        public String Address { get; set; }
        /// <summary>
        /// Aktuális üzenet lekérdezése, vagy beállítása.
        /// </summary>
        public String CurrentMessage { get; set; }

        /// <summary>
        /// Üzenetküldés parancsának lekérdezése.
        /// </summary>
        public ICommand SendCommand
        {
            get
            {
                if (_SendCommand == null) // ha még nem létezik
                {
                    _SendCommand = new RelayCommand(param => _Manager.Connected, param =>
                    {
                        String message = UserName + " [" + DateTime.Now.ToString("hh:mm:ss") + "]: " + CurrentMessage;
                        Dispatcher.CurrentDispatcher.BeginInvoke(new Action(() => InsertMessage(message)));
                        _Manager.Send(message);
                    });
                    // létrehozzuk a parancsot úgy, hogy a megadott metódusokat használja fel
                }

                return _SendCommand;
            }
        }

        /// <summary>
        /// Kapcsolódás parancsának lekérdezése.
        /// </summary>
        public ICommand ConnectCommand
        {
            get
            {
                if (_ConnectCommand == null) // ha még nem létezik
                {
                    _ConnectCommand = new RelayCommand(param => !_Manager.Connected, param =>
                    {
                        _Manager.Connect(Address, 399);
                    });
                }

                return _ConnectCommand;
            }
        }

        public NetworkManagerViewModel()
        {
            Messages = new ObservableCollection<String>();
            _Manager = new NetworkManager();
            _Manager.ConnectionChanged += new EventHandler(Manager_ConnectionChanged);
            _Manager.MessageReceived += new EventHandler(Manager_MessageReceived);
            _Manager.OperationFailed += new EventHandler(Manager_OperationFailed);

            CurrentMessage = "Hello";
            Address = "127.0.0.1";
            UserName = "Anonymus";
        }

        private void Manager_OperationFailed(object sender, EventArgs e)
        {
            Dispatcher.CurrentDispatcher.BeginInvoke(new Action(() => InsertMessage("Operation failed! Terminating connection...")));
            // szálbiztosan írjuk vissza az üzenetet a gyűjteménybe, különben nem működne az adatkapcsolás
        }

        private void Manager_MessageReceived(object sender, EventArgs e)
        {
            Dispatcher.CurrentDispatcher.BeginInvoke(new Action(() => InsertMessage(_Manager.CurrentMessage)));
        }

        private void Manager_ConnectionChanged(object sender, EventArgs e)
        {
            OnPropertyChanged("Connected");
        }

        private void InsertMessage(String message)
        {
            Messages.Insert(0, message);
            OnPropertyChanged("Messages"); 
        }
    }
}
