﻿using System;
using System.Net.Sockets;

namespace ELTE.Net.ChatRoom.Tcp.Client
{
    /// <summary>
    /// Klienskezelő típusa.
    /// </summary>
    public class ChatClient
    {
        private ChatServer _Server;
        private Socket _MessageSocket;

        /// <summary>
        /// Klienskezelő példányosítása.
        /// </summary>
        /// <param name="socket">Hálózati csatoló.</param>
        /// <param name="server">Szerverkezelő.</param>
        public ChatClient(Socket socket, ChatServer server)
        {
            _Server = server;
            _MessageSocket = socket;

            SocketAsyncEventArgs socketAsyncEventArgs = new SocketAsyncEventArgs();
            socketAsyncEventArgs.RemoteEndPoint = _MessageSocket.RemoteEndPoint;
            socketAsyncEventArgs.SetBuffer(new Byte[1024], 0, 1024);
            socketAsyncEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Receive_Completed);
            _MessageSocket.ReceiveAsync(socketAsyncEventArgs);
        }

        private void Receive_Completed(object server, SocketAsyncEventArgs e)
        {
            if (e.SocketError == SocketError.Success)
            {
                Byte[] messageBytes = new Byte[e.BytesTransferred]; // megnézzük a fogadott adatokat
                Array.Copy(e.Buffer, messageBytes, e.BytesTransferred);
                _Server.Broadcast(messageBytes); // elküldjük mindenkinek

                e.Dispose();

                SocketAsyncEventArgs socketAsyncEventArgs = new SocketAsyncEventArgs(); // új üzenet fogadása
                socketAsyncEventArgs.RemoteEndPoint = _MessageSocket.RemoteEndPoint;
                socketAsyncEventArgs.SetBuffer(new Byte[1024], 0, 1024);
                socketAsyncEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Receive_Completed);
                _MessageSocket.ReceiveAsync(socketAsyncEventArgs);
            }
        }

        /// <summary>
        /// Üzenet küldése.
        /// </summary>
        /// <param name="messageBytes">Üzenet byte-tömbként.</param>
        public void SendMessage(Byte[] messageBytes)
        {
            _MessageSocket.Send(messageBytes);
        }
    }
}
