﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;

namespace ELTE.Net.ChatRoom.Tcp.Client
{
    /// <summary>
    /// Szerverkezelő típusa.
    /// </summary>
    public class ChatServer
    {
        private Socket _Listener;
        private String _ServerName;
        private List<IPAddress> _ServerAddressList;
        private Int32 _ServerPort;
        private List<ChatClient> _ClientList;

        /// <summary>
        /// Szerverkezelő példányosítása.
        /// </summary>
        public ChatServer()
        {
            _ServerAddressList = new List<IPAddress>();
            _ClientList = new List<ChatClient>();
            _ServerPort = 399;

            // a szervernek lehet több IP címe is, ezeket a DNS-en keresztül érjük el:
            try
            {
                _ServerName = Dns.GetHostName(); // a szerver DNS nevének lekérdezése
                IPHostEntry ipEntry = Dns.GetHostEntry(_ServerName); // a szerver nevének megfelelő címek lekérdezése
                foreach (IPAddress address in ipEntry.AddressList)
                {
                    if (address.AddressFamily == AddressFamily.InterNetwork) // ha ez IPv4 cím
                    {
                        _ServerAddressList.Add(address); // akkor felvesszük a lehetséges címek közé
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error occured: ", ex.Message);
            }
        }

        /// <summary>
        /// Szerver futtatása.
        /// </summary>
        public void Run()
        {
            // ha sikerült valamit kiolvasni:
            if (_ServerAddressList == null || _ServerAddressList.Count < 1)
            {
                Console.WriteLine("Error occured: unable to get local address.");
            }
            else
            {
                Console.WriteLine("Chat server started...");
                Console.WriteLine("Listening on : [{0}] {1}:{2}.", _ServerName, _ServerAddressList[0], _ServerPort);

                _Listener = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                _Listener.Bind(new IPEndPoint(_ServerAddressList[0], _ServerPort)); // csatolás IP címhez
                _Listener.Listen(10); // maximum 10 kapcsolat

                SocketAsyncEventArgs socketAsyncEventArgs = new SocketAsyncEventArgs();
                socketAsyncEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Connect_Completed);
                _Listener.AcceptAsync(socketAsyncEventArgs); // fogadás aszinkron módon
            }
        }

        /// <summary>
        /// Szerver leállítása.
        /// </summary>
        public void Stop()
        {
            _Listener.Close();
            GC.WaitForPendingFinalizers(); // a szemétgyűjtő várjon a megfelelő objektumok megsemmisülésére
        }

        private void Connect_Completed(object sender, SocketAsyncEventArgs e)
        {
            NewConnection(e.AcceptSocket); // a paraméterből lekérhetjük a csatolót

            e.Dispose();

            SocketAsyncEventArgs socketAsyncEventArgs = new SocketAsyncEventArgs();
            socketAsyncEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(Connect_Completed);
            _Listener.AcceptAsync(socketAsyncEventArgs); // további csatlakozás fogadása
        }

        /// <summary>
        /// Új kapcsolat létrehozása.
        /// </summary>
        /// <param name="clientSocket"></param>
        private void NewConnection(Socket clientSocket)
        {
            ChatClient client = new ChatClient(clientSocket, this);
            _ClientList.Add(client);
            Console.WriteLine("Client joined at IP " + clientSocket.RemoteEndPoint);

            String message = "[SERVER] Connection successful, You joined at " + DateTime.Now.ToString("G");
            Byte[] messageBytes = System.Text.Encoding.UTF8.GetBytes(message); // átalakítás bájt tömbbé
            client.SendMessage(messageBytes); // elküldés a kliensnek
        }
        
        /// <summary>
        /// Üzenetküldés minden kliens számára.
        /// </summary>
        /// <param name="messageBytes">Üzenet byte-tömbként.</param>
        public void Broadcast(Byte[] messageBytes)
        {
            foreach (ChatClient client in _ClientList)
                client.SendMessage(messageBytes);
        }
    }
}
