﻿using System;
using System.Xml.Serialization;

namespace ELTE.Net.GameServer.Shared
{
    /// <summary>
    /// Játékállapot felsoroló típusa.
    /// </summary>
    public enum GamePhase { Open, Playing, Completed }

    /// <summary>
    /// Játékadatok típusa.
    /// </summary>
    [XmlInclude(typeof(Player))] // beágyazott típus szérializációja
    public class Game
    {
        /// <summary>
        /// Játék azonosítójának lekérdezése, vagy beállítása.
        /// </summary>
        [XmlAttribute]
        public Int32 GameId { get; set; }
        /// <summary>
        /// Játék létrehozási idejének lekérdezése, vagy beállítása.
        /// </summary>
        [XmlElement]
        public DateTime CreateTime { get; set; }
        /// <summary>
        /// Játék kezdési idejének lekérdezése, vagy beállítása.
        /// </summary>
        [XmlElement]
        public DateTime? StartTime { get; set; }
        /// <summary>
        /// Játék befejezési idejének lekérdezése, vagy beállítása.
        /// </summary>
        [XmlElement]
        public DateTime? EndTime { get; set; }
        /// <summary>
        /// Első játékos lekérdezése, vagy beállítása.
        /// </summary>
        [XmlElement]
        public Player FirstPlayer { get; set; }
        /// <summary>
        /// Második játékos lekérdezése, vagy beállítása.
        /// </summary>
        [XmlElement]
        public Player SecondPlayer { get; set; }
        /// <summary>
        /// Győztes lekérdezése, vagy beállítása.
        /// </summary>
        [XmlElement]
        public Player Winner { get; set; }

        /// <summary>
        /// Játékfázis lekérdezése.
        /// </summary>
        public GamePhase Phase 
        {
            get {
                if (EndTime != null)
                    return GamePhase.Completed;
                if (StartTime != null)
                    return GamePhase.Playing;
                else
                    return GamePhase.Open;
            }
        }
    }
}
