﻿using System;
using System.IO;
using System.Xml.Serialization;

namespace ELTE.Net.GameServer.Shared
{
    /// <summary>
    /// Üzenetkód felsoroló típusa.
    /// </summary>
    public enum MessageCode 
    { 
        Login, CreateGame, JoinGame, JoinAccepted, JoinRejected, 
        EndGame, ChangeGameState, GetOpenGames 
    }

    /// <summary>
    /// Üzenet típusa.
    /// </summary>
    [XmlRoot]
    [XmlInclude(typeof(Game)), XmlInclude(typeof(Game[]))] // a beágyaztható típusoknak is XML szérializálhatóaknak kell lennie
    public class Message
    {
        /// <summary>
        /// Üzenet kódjának lekérdezése, vagy beállítása.
        /// </summary>
        [XmlAttribute]
        public MessageCode Code { get; set; }
        /// <summary>
        /// Üzenet tartalmának lekérdezése, vagy beállítása.
        /// </summary>
        [XmlElement]
        public Object Content { get; set; }

        /// <summary>
        /// Üzenet XML alapú szérializációja.
        /// </summary>
        /// <param name="code">Üzenet kódja.</param>
        /// <param name="param">Üzenet tartalma.</param>
        /// <returns>Az üzenet byte tömbje.</returns>
        public static Byte[] Serialize(MessageCode code, Object param)
        {
            Message message = new Message { Code = code, Content = param };

            using (MemoryStream stream = new MemoryStream())
            {
                XmlSerializer serializer = new XmlSerializer(typeof(Message));
                serializer.Serialize(stream, message);
                return stream.ToArray();
            }
        }
        /// <summary>
        /// Üzenet visszaállítása XML-alapú szérializációból.
        /// </summary>
        /// <param name="buffer">Az üzenet byte tömbje</param>
        /// <param name="length">Az üzenet hossza.</param>
        /// <returns>A visszaállított üzenet.</returns>
        public static Message Deserialize(Byte[] buffer, Int32 length)
        {
            using (MemoryStream stream = new MemoryStream(buffer, 0, length))
            {
                XmlSerializer serializer = new XmlSerializer(typeof(Message));
                Message message = serializer.Deserialize(stream) as Message;
                return message;
            }
        }
    }
}
